package com.sd.business.service.artwork.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.artwork.dto.ArtworkLibraryDto;
import com.sd.business.entity.artwork.dto.ArtworkLibrarySelectDto;
import com.sd.business.entity.artwork.po.ArtworkLibrary;
import com.sd.business.entity.artwork.vo.ArtworkLibraryVo;
import com.sd.business.mapper.artwork.ArtworkLibraryMapper;
import com.sd.business.service.artwork.ArtworkLibraryService;
import com.sd.framework.util.Assert;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 图稿库 服务实现类
 * </p>
 *
 * @author
 * @since 2023-08-02
 */
@Service
public class ArtworkLibraryServiceImpl extends ServiceImpl<ArtworkLibraryMapper, ArtworkLibrary> implements ArtworkLibraryService {

    @Override
    public List<ArtworkLibraryVo> getList(ArtworkLibrarySelectDto dto) {
        IWrapper<ArtworkLibrary> wrapper = getWrapper();
        wrapper.orderByDesc("al", ArtworkLibrary::getId);
        List<ArtworkLibraryVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public Page<ArtworkLibraryVo> getPage(ArtworkLibrarySelectDto dto) {
        IWrapper<ArtworkLibrary> wrapper = getWrapper();
        wrapper.orderByDesc("al", ArtworkLibrary::getId);
        Page<ArtworkLibraryVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public ArtworkLibraryVo detail(Long id) {
        ArtworkLibrary ArtworkLibrary = this.getById(id);
        ArtworkLibraryVo result = BeanUtil.toBean(ArtworkLibrary, ArtworkLibraryVo.class);
        return result;
    }

    /**
     * 新增图稿
     *
     * @param artworkLibraryDto
     */
    @Override
    public void add(ArtworkLibraryDto artworkLibraryDto) {
        long count = count(q -> q.eq(ArtworkLibrary::getArtworkName, artworkLibraryDto.getArtworkName()).last("limit 1"));
        Assert.eqZero(count, "图稿名称已存在");
        this.save(artworkLibraryDto);
    }

    /**
     * 修改图稿
     *
     * @param artworkLibraryDto
     */
    @Override
    public void edit(ArtworkLibraryDto artworkLibraryDto) {
        long count = count(q -> q
                .eq(ArtworkLibrary::getArtworkName, artworkLibraryDto.getArtworkName())
                .ne(BaseIdPo::getId, artworkLibraryDto.getId())
                .last("limit 1"));

        Assert.eqZero(count, "图稿名称已存在");
        this.updateById(artworkLibraryDto);
    }

    /**
     * 删除图稿
     *
     * @param id
     */
    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

}
