package com.sd.business.service.statement.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.file.entity.FileInfoVo;
import com.fjhx.file.utils.ObsFileUtil;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.order.po.OrderInfo;
import com.sd.business.entity.order.po.OrderSku;
import com.sd.business.entity.order.po.OrderSkuBom;
import com.sd.business.entity.statement.dto.FileUploadDto;
import com.sd.business.entity.statement.dto.StatementOfAccountDto;
import com.sd.business.entity.statement.dto.StatementOfAccountSelectDto;
import com.sd.business.entity.statement.po.StatementOfAccount;
import com.sd.business.entity.statement.vo.DocumentByBomVo;
import com.sd.business.entity.statement.vo.DocumentBySkuVo;
import com.sd.business.entity.statement.vo.ReconciliationDetailVo;
import com.sd.business.entity.statement.vo.StatementOfAccountVo;
import com.sd.business.mapper.statement.StatementOfAccountMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.business.service.department.DepartmentService;
import com.sd.business.service.order.OrderService;
import com.sd.business.service.order.OrderSkuBomService;
import com.sd.business.service.order.OrderSkuService;
import com.sd.business.service.sku.SkuSpecService;
import com.sd.business.service.statement.StatementOfAccountService;
import com.sd.business.util.CodeEnum;
import com.sd.framework.util.Assert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;


/**
 * <p>
 * 对账单 服务实现类
 * </p>
 *
 * @author
 * @since 2023-07-31
 */
@Service
public class StatementOfAccountServiceImpl extends ServiceImpl<StatementOfAccountMapper, StatementOfAccount> implements StatementOfAccountService {

    @Autowired
    private OrderService orderService;

    @Autowired
    private OrderSkuService orderSkuService;

    @Autowired
    private OrderSkuBomService orderSkuBomService;

    @Autowired
    private DepartmentService departmentService;

    @Autowired
    private SkuSpecService skuSpecService;

    @Autowired
    private BomSpecService bomSpecService;

    @Override
    public Page<StatementOfAccountVo> getPage(StatementOfAccountSelectDto dto) {

        IWrapper<StatementOfAccount> wrapper = getWrapper();
        wrapper.like("soa", StatementOfAccount::getCode, dto.getCode());
        wrapper.eq("soa", StatementOfAccount::getDepartmentId, dto.getDepartmentId());
        wrapper.ge("soa", StatementOfAccount::getCreateTime, dto.getBeginTime());
        wrapper.le("soa", StatementOfAccount::getCreateTime, dto.getEndTime());
        wrapper.orderByDesc("soa", StatementOfAccount::getId);

        Page<StatementOfAccountVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<StatementOfAccountVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        // 查询对账单和转帐凭证
        List<Long> idList = records.stream().map(BaseIdPo::getId).collect(Collectors.toList());
        Map<Long, List<FileInfoVo>> receiptFileMap = ObsFileUtil.getFileMap(idList, 1);
        Map<Long, List<FileInfoVo>> proofFileMap = ObsFileUtil.getFileMap(idList, 2);

        Map<Long, Map<String, Object>> map = orderService.listMaps(Wrappers.<OrderInfo>query()
                        .select(
                                "statement_of_account_id as statementOfAccountId",
                                "sum(total_amount) as amount",
                                "count(0) as orderNum"
                        )
                        .in("statement_of_account_id", idList)
                        .groupBy("statement_of_account_id")
                )
                .stream()
                .collect(Collectors.toMap(
                        item -> Convert.toLong(item.get("statementOfAccountId")),
                        Function.identity()
                ));

        for (StatementOfAccountVo record : records) {
            record.setReceiptFileList(receiptFileMap.getOrDefault(record.getId(), Collections.emptyList()));
            record.setProofFileList(proofFileMap.getOrDefault(record.getId(), Collections.emptyList()));
            Map<String, Object> tempMap = map.get(record.getId());
            if (tempMap == null) {
                record.setOrderNum(0);
                record.setAmount(BigDecimal.ZERO);
            } else {
                record.setOrderNum(Convert.toInt(tempMap.get("orderNum"), 0));
                record.setAmount(Convert.toBigDecimal(tempMap.get("amount"), BigDecimal.ZERO));
            }
        }

        return page;
    }

    @Override
    public List<ReconciliationDetailVo> detail(Long id) {

        List<OrderInfo> orderInfoList = orderService.list(q -> q.eq(OrderInfo::getStatementOfAccountId, id));

        List<ReconciliationDetailVo> result = orderInfoList.stream().map(item -> ReconciliationDetailVo.builder()
                .orderId(item.getId())
                .departmentId(item.getDepartmentId())
                .orderCode(item.getCode())
                .totalAmount(item.getTotalAmount())
                .productTotalAmount(item.getProductTotalAmount())
                .customProcessingFee(item.getCustomProcessingFee())
                .lssueFee(item.getLssueFee())
                .deliveryMaterialsFee(item.getDeliveryMaterialsFee())
                .packingLabor(item.getPackingLabor())
                .packagingMaterialCost(item.getPackagingMaterialCost())
                .build()
        ).collect(Collectors.toList());

        departmentService.attributeAssign(result, ReconciliationDetailVo::getDepartmentId, (item, department) -> {
            item.setDepartmentName(department.getName());
        });

        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(StatementOfAccountDto statementOfAccountDto) {

        List<Long> orderIdList = statementOfAccountDto.getOrderIdList();

        statementOfAccountDto.setCode(CodeEnum.STATEMENT_OF_ACCOUNT_CODE.getCode());
        statementOfAccountDto.setType(1);
        this.save(statementOfAccountDto);

        if (ObjectUtil.isNotEmpty(orderIdList)) {
            orderService.update(q -> q
                    .in(BaseIdPo::getId, orderIdList)
                    .set(OrderInfo::getStatementOfAccountId, statementOfAccountDto.getId())
            );
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(StatementOfAccountDto statementOfAccountDto) {
        Long statementOfAccountId = statementOfAccountDto.getId();
        List<Long> orderIdList = statementOfAccountDto.getOrderIdList();

        Assert.notNull(statementOfAccountId, "对账单id不能为空");

        if (ObjectUtil.isNotEmpty(orderIdList)) {
            orderService.update(q -> q
                    .in(BaseIdPo::getId, orderIdList)
                    .set(OrderInfo::getStatementOfAccountId, statementOfAccountId)
            );
        }

        orderService.update(q -> q
                .eq(OrderInfo::getStatementOfAccountId, statementOfAccountId)
                .notIn(ObjectUtil.isNotEmpty(orderIdList), BaseIdPo::getId, orderIdList)
                .set(OrderInfo::getStatementOfAccountId, null)
        );

    }

    @DSTransactional
    @Override
    public void delete(Long id) {

        orderService.update(q -> q
                .eq(OrderInfo::getStatementOfAccountId, id)
                .set(OrderInfo::getStatementOfAccountId, null)
        );

        this.removeById(id);
        ObsFileUtil.removeFile(id);
    }

    @Override
    public void editFile(FileUploadDto dto, int type) {
        ObsFileUtil.editFile(dto.getFileList(), dto.getId(), type);
    }

    @Override
    public List<DocumentBySkuVo> getDocumentBySku(Long statementOfAccountId) {

        // 获取订单id列表
        List<Long> ordedrIdList = getOrderIdList(statementOfAccountId);

        if (ordedrIdList.size() == 0) {
            return Collections.emptyList();
        }

        // 获取订单sku
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, ordedrIdList));

        // 生成结果集
        List<DocumentBySkuVo> documentBySkuVoList = orderSkuList.stream()
                .map(item -> DocumentBySkuVo.builder()
                        .skuSpecId(item.getSkuId())
                        .quantity(item.getQuantity())
                        .unitPrice(item.getUnitPrice()
                                .add(item.getCustomProcessingFee())
                                .add(item.getLssueFee())
                                .add(item.getDeliveryMaterialsFee())
                                .add(item.getPackingLabor())
                                .add(item.getPackagingMaterialCost()))
                        .build())
                .peek(item -> item.setSubtotal(item.getQuantity().multiply(item.getUnitPrice())))
                .peek(item -> item.setTotal(item.getSubtotal()))
                .collect(Collectors.toList());

        // 赋值sku规格品名和品号
        skuSpecService.attributeAssign(documentBySkuVoList, DocumentBySkuVo::getSkuSpecId, (item, skuSpec) -> {
            item.setSkuSpecCode(skuSpec.getCode());
            item.setSkuSpecName(skuSpec.getName());
        });

        // 合并单价、小计、sku规格id系统的对账数据
        Collection<DocumentBySkuVo> documentBySkuVoCollection = documentBySkuVoList.stream()
                .sorted(Comparator.comparing(DocumentBySkuVo::getSkuSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getSkuSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v2.getQuantity());
                            v1.setSubtotal(v2.getSubtotal());
                            v1.setTotal(v2.getTotal());
                            return v1;
                        }
                )).values();

        return new ArrayList<>(documentBySkuVoCollection);
    }

    @Override
    public List<DocumentByBomVo> getDocumentByBom(Long statementOfAccountId) {

        // 获取订单id列表
        List<Long> ordedrIdList = getOrderIdList(statementOfAccountId);

        if (ordedrIdList.size() == 0) {
            return Collections.emptyList();
        }

        // 获取订单sku
        List<OrderSku> orderSkuList = orderSkuService.list(q -> q.in(OrderSku::getOrderId, ordedrIdList));

        // 主材bom
        List<DocumentByBomVo> result = orderSkuList.stream()
                .map(item -> DocumentByBomVo.builder()
                        .bomSpecId(item.getBomSpecId())
                        .quantity(item.getQuantity())
                        .unitPrice(item.getUnitPrice())
                        .laserLogoSummary((Objects.equals(item.getCustomProcessingType(), "20")
                                ? item.getCustomProcessingFee() : BigDecimal.ZERO).multiply(item.getQuantity()))
                        .laserMitochondrialSummary((Objects.equals(item.getCustomProcessingType(), "10")
                                ? item.getCustomProcessingFee() : BigDecimal.ZERO).multiply(item.getQuantity()))
                        .lssueFeeSummary(item.getLssueFee().multiply(item.getQuantity()))
                        .deliveryMaterialsFeeSummary(item.getDeliveryMaterialsFee().multiply(item.getQuantity()))
                        .packingLaborSummary(item.getPackingLabor().multiply(item.getQuantity()))
                        .build())
                .peek(item -> item.setSubtotal(item.getUnitPrice().multiply(item.getQuantity())
                        .add(item.getLaserLogoSummary())
                        .add(item.getLaserMitochondrialSummary())
                        .add(item.getLssueFeeSummary())
                        .add(item.getDeliveryMaterialsFeeSummary())
                        .add(item.getPackingLaborSummary())))
                .peek(item -> item.setTotal(item.getSubtotal()))
                .collect(Collectors.toList());

        // 赋值主材bom品名品号
        bomSpecService.attributeAssign(result, DocumentByBomVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        // 合并同加个同规格主材bom，数量相加
        Collection<DocumentByBomVo> documentByBomVoCollection = result.stream()
                .sorted(Comparator.comparing(DocumentByBomVo::getBomSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getBomSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setLaserLogoSummary(v1.getLaserLogoSummary().add(v2.getLaserLogoSummary()));
                            v1.setLaserMitochondrialSummary(v1.getLaserMitochondrialSummary().add(v2.getLaserMitochondrialSummary()));
                            v1.setLssueFeeSummary(v1.getLssueFeeSummary().add(v2.getLssueFeeSummary()));
                            v1.setDeliveryMaterialsFeeSummary(v1.getDeliveryMaterialsFeeSummary().add(v2.getDeliveryMaterialsFeeSummary()));
                            v1.setPackingLaborSummary(v1.getPackingLaborSummary().add(v2.getPackingLaborSummary()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getSubtotal());
                            return v1;
                        }
                )).values();

        // 赋值合并后结果
        result = new ArrayList<>(documentByBomVoCollection);

        // 包材bom
        List<OrderSkuBom> orderSkuBomList = orderSkuBomService.list(q -> q.in(OrderSkuBom::getOrderId, ordedrIdList));
        List<DocumentByBomVo> bomVoList = orderSkuBomList.stream().map(item ->
                DocumentByBomVo.builder()
                        .bomSpecId(item.getBomSpecId())
                        .quantity(item.getQuantity())
                        .unitPrice(item.getUnitPrice())
                        .subtotal(item.getUnitPrice().multiply(item.getQuantity()))
                        .total(item.getUnitPrice().multiply(item.getQuantity()))
                        .build()
        ).collect(Collectors.toList());

        // 赋值包材bom品名品号
        bomSpecService.attributeAssign(bomVoList, DocumentByBomVo::getBomSpecId, (item, bomSpec) -> {
            item.setBomSpecCode(bomSpec.getCode());
            item.setBomSpecName(bomSpec.getName());
        });

        // 合并同加个同规格包材bom，数量相加
        Collection<DocumentByBomVo> bomValues = bomVoList.stream()
                .sorted(Comparator.comparing(DocumentByBomVo::getBomSpecCode))
                .collect(Collectors.toMap(
                        item -> item.getUnitPrice() + ":" + item.getBomSpecId(),
                        Function.identity(),
                        (v1, v2) -> {
                            v1.setQuantity(v1.getQuantity().add(v2.getQuantity()));
                            v1.setSubtotal(v1.getSubtotal().add(v2.getSubtotal()));
                            v1.setTotal(v1.getSubtotal());
                            return v1;
                        }
                )).values();

        // 合并主材包材
        result.addAll(bomValues);

        return result;
    }


    /**
     * 根据对账单id获取对账订单id列表
     */
    private List<Long> getOrderIdList(Long statementOfAccountId) {
        Assert.notNull(statementOfAccountId, "对账单id不能为空");
        List<OrderInfo> orderList = orderService.list(q -> q.eq(OrderInfo::getStatementOfAccountId, statementOfAccountId));
        return orderList.stream().map(BaseIdPo::getId).collect(Collectors.toList());
    }

}
